@@ -29,14 +29,14 @@ use vars qw(@ISA @EXPORT %EXPORT_TAGS $VERSION %SubTests @SkipReasons);
              have_cgi have_access have_auth have_module have_apache
              have_min_apache_version have_apache_version have_perl 
              have_min_perl_version have_min_module_version
-             have_threads under_construction have_apache_mpm);
+             have_threads under_construction skip_reason have_apache_mpm);
 
 # everything but ok(), skip(), and plan() - Test::More provides these
 my @test_more_exports = grep { ! /^(ok|skip|plan)$/ } @EXPORT;
 
 %EXPORT_TAGS = (withtestmore => \@test_more_exports);
 
-$VERSION = '1.10';
+$VERSION = '1.12';
 
 %SubTests = ();
 @SkipReasons = ();
@@ -165,6 +165,7 @@ sub plan {
             my $reason = join ', ',
               @SkipReasons ? @SkipReasons : "no reason given";
             print "1..0 # skipped: $reason\n";
+            @SkipReasons = (); # reset
             exit; #XXX: Apache->exit
         }
     }
@@ -391,7 +392,13 @@ sub under_construction {
     return 0;
 }
 
-# normalize Apache-sytle version strings (2.0.48, 0.9.4)
+sub skip_reason {
+    my $reason = shift || 'no reason specified';
+    push @SkipReasons, $reason;
+    return 0;
+}
+
+# normalize Apache-style version strings (2.0.48, 0.9.4)
 # for easy numeric comparison.  note that 2.1 and 2.1.0
 # are considered equivalent.
 sub normalize_vstring {
@@ -537,7 +544,9 @@ exported.
 
 Functions that can be used as a last argument to the extended plan():
 
-=over have_http11
+=over 
+
+=item have_http11
 
   plan tests => 5, &have_http11;
 
@@ -711,6 +720,23 @@ reference, but be careful that the keys will be different.
 
 Also see plan().
 
+=item under_construction
+
+  plan tests => 5, under_construction;
+
+skip all tests, noting that the tests are under construction
+
+=item skip_reason
+
+  plan tests => 5, skip_reason('my custom reason');
+
+skip all tests.  the reason you specify will be given at runtime.
+if no reason is given a default reason will be used.
+
+=back
+
+=head1 Additional Configuration Variables
+
 =item config
 
   my $cfg = Apache::Test::config();
@@ -734,8 +760,6 @@ If no arguments are passed, the reference to the variables hash is
 returned. If one or more arguments are passed the corresponding values
 are returned.
 
-=back
-
 =head1 Test::More Integration
 
 There are a few caveats if you want to use I<Apache::Test> with 
