@@ -272,6 +272,15 @@ sub inherit_config_file_or_directory {
     while (<$fh>) {
         s/^\s*//; s/\s*$//; s/^\#.*//;
         next if /^$/;
+
+        # support continuous config lines (which use \ to break the line)
+        while (s/\\$//) {
+            my $cont = <$fh>;
+            $cont =~ s/^\s*//;
+            $cont =~ s/\s*$//;
+            $_ .= $cont;
+        }
+
         (my $directive, $_) = split /\s+/, $_, 2;
 
         if ($directive eq "Include") {
@@ -302,6 +311,7 @@ sub inherit_config {
       catfile $self->{httpd_basedir}, 'htdocs';
 
     my $file = $self->{vars}->{httpd_conf};
+    my $extra_file = $self->{vars}->{httpd_conf_extra};
 
     unless ($file and -e $file) {
         if (my $base = $self->{httpd_basedir}) {
@@ -313,7 +323,16 @@ sub inherit_config {
         }
     }
 
-    return unless $file;
+    unless ($extra_file and -e $extra_file) {
+        if ($extra_file and my $base = $self->{httpd_basedir}) {
+            my $default_conf = catfile qw(conf $extra_file);
+            $extra_file = catfile $base, $default_conf;
+            # SERVER_CONFIG_FILE might be an absolute path
+            $extra_file = $default_conf if !-e $extra_file and -e $default_conf;
+        }
+    }
+
+    return unless $file or $extra_file;
 
     my $c = $self->{inherit_config};
 
@@ -324,7 +343,8 @@ sub inherit_config {
         }
     }
 
-    $self->inherit_config_file_or_directory($file);
+    $self->inherit_config_file_or_directory($file) if $file;
+    $self->inherit_config_file_or_directory($extra_file) if $extra_file;
 
     #apply what we parsed
     while (my($spec, $wanted) = each %wanted_config) {
@@ -344,6 +364,7 @@ sub get_httpd_static_modules {
     my $httpd = $self->{vars}->{httpd};
     return unless $httpd;
 
+    $httpd = shell_ready($httpd);
     my $cmd = "$httpd -l";
     my $list = $self->open_cmd($cmd);
 
@@ -364,6 +385,7 @@ sub get_httpd_defines {
     my $httpd = $self->{vars}->{httpd};
     return unless $httpd;
 
+    $httpd = shell_ready($httpd);
     my $cmd = "$httpd -V";
     my $proc = $self->open_cmd($cmd);
 
@@ -415,6 +437,7 @@ sub httpd_version {
     return unless $httpd;
 
     my $version;
+    $httpd = shell_ready($httpd);
     my $cmd = "$httpd -v";
 
     my $v = $self->open_cmd($cmd);
