@@ -29,7 +29,7 @@ use Cwd;
 use ExtUtils::MakeMaker;
 use File::Find qw(finddepth);
 use File::Path;
-use File::Spec::Functions qw(catfile catdir);
+use File::Spec::Functions qw(catfile catdir canonpath);
 use File::Basename qw(basename dirname);
 use Getopt::Long qw(GetOptions);
 use Config;
@@ -49,7 +49,8 @@ my %core_files  = ();
 my %original_t_perms = ();
 
 my @std_run      = qw(start-httpd run-tests stop-httpd);
-my @others       = qw(verbose configure clean help ssl http11 bugreport save);
+my @others       = qw(verbose configure clean help ssl http11 bugreport 
+                      save no-httpd);
 my @flag_opts    = (@std_run, @others);
 my @string_opts  = qw(order trace);
 my @ostring_opts = qw(proxy ping);
@@ -65,23 +66,28 @@ my %usage = (
    'start-httpd'     => 'start the test server',
    'run-tests'       => 'run the tests',
    'times=N'         => 'repeat the tests N times',
-   'order=mode'      => 'run the tests in one of the modes: (repeat|rotate|random|SEED)',
+   'order=mode'      => 'run the tests in one of the modes: ' .
+                        '(repeat|rotate|random|SEED)',
    'stop-httpd'      => 'stop the test server',
+   'no-httpd'        => 'run the tests without configuring or starting httpd',
    'verbose[=1]'     => 'verbose output',
-   'configure'       => 'force regeneration of httpd.conf (tests will not be run)',
+   'configure'       => 'force regeneration of httpd.conf ' .
+                        ' (tests will not be run)',
    'clean'           => 'remove all generated test files',
    'help'            => 'display this message',
    'bugreport'       => 'print the hint how to report problems',
    'preamble'        => 'config to add at the beginning of httpd.conf',
    'postamble'       => 'config to add at the end of httpd.conf',
    'ping[=block]'    => 'test if server is running or port in use',
-   'debug[=name]'    => 'start server under debugger name (e.g. gdb, ddd, ...)',
+   'debug[=name]'    => 'start server under debugger name (gdb, ddd, etc.)',
    'breakpoint=bp'   => 'set breakpoints (multiply bp can be set)',
-   'header'          => "add headers to (".join('|', @request_opts).") request",
+   'header'          => "add headers to (" .
+                         join('|', @request_opts) . ") request",
    'http11'          => 'run all tests with HTTP/1.1 (keep alive) requests',
    'ssl'             => 'run tests through ssl',
    'proxy'           => 'proxy requests (default proxy is localhost)',
-   'trace=T'         => 'change tracing default to: warning, notice, info, debug, ...',
+   'trace=T'         => 'change tracing default to: warning, notice, ' .
+                        'info, debug, ...',
    'save'            => 'save test paramaters into Apache::TestConfigData',
    (map { $_, "\U$_\E url" } @request_opts),
 );
@@ -146,8 +152,9 @@ sub split_test_args {
     my @leftovers = ();
     for (@$argv) {
         my $arg = $_;
-        #need the t/ for stat-ing, but don't want to include it in test output
-        $arg =~ s@^(?:\./)?t/@@;
+        # need the t/ (or t\) for stat-ing, but don't want to include
+        # it in test output
+        $arg =~ s@^(?:\.[\\/])?t[\\/]@@;
         my $file = catfile $t_dir, $arg;
         if (-d $file and $_ ne '/') {
             my @files = <$file/*.t>;
@@ -180,7 +187,7 @@ sub split_test_args {
         push @leftovers, $_;
     }
 
-    $self->{tests} = \@tests;
+    $self->{tests} = [ map { canonpath($_) } @tests ];
     $self->{argv}  = \@leftovers;
 }
 
@@ -190,7 +197,8 @@ sub die_on_invalid_args {
     # at this stage $self->{argv} should be empty
     my @invalid_argv = @{ $self->{argv} };
     if (@invalid_argv) {
-        error "unknown opts or test names: @invalid_argv\n-help will list options\n";
+        error "unknown opts or test names: @invalid_argv\n" .
+            "-help will list options\n";
         exit_perl 0;
     }
 
@@ -323,7 +331,8 @@ sub default_run_opts {
 
     unless (grep { exists $opts->{$_} } @std_run, @request_opts) {
         if (@$tests && $self->{server}->ping) {
-            #if certain tests are specified and server is running, dont restart
+            # if certain tests are specified and server is running,
+            # dont restart
             $opts->{'run-tests'} = 1;
         }
         else {
@@ -347,6 +356,12 @@ sub install_sighandlers {
 
     $SIG{__DIE__} = sub {
         return unless $_[0] =~ /^Failed/i; #dont catch Test::ok failures
+
+        # _show_results() calls die() under a few conditions, such as
+        # when no tests are run or when tests fail.  make sure the message
+        # is propagated back to the user.
+        print $_[0] if (caller(1))[3]||'' eq 'Test::Harness::_show_results';
+
         $server->stop(1) if $opts->{'start-httpd'};
         $server->failed_msg("error running tests");
         exit_perl 0;
@@ -365,19 +380,18 @@ sub install_sighandlers {
     #try to make sure we scan for core no matter what happens
     #must eval "" to "install" this END block, otherwise it will
     #always run, a subclass might not want that
-
     eval 'END {
-             return unless is_parent(); # because of fork
-             $self ||=
-                 Apache::TestRun->new(test_config => Apache::TestConfig->thaw);
-             {
-                 local $?; # preserve the exit status
-                 eval {
-                    $self->scan_core;
-                 };
-             }
-             $self->try_bug_report();
-         }';
+        return unless is_parent(); # because of fork
+        $self ||=
+            Apache::TestRun->new(test_config => Apache::TestConfig->thaw);
+        {
+            local $?; # preserve the exit status
+            eval {
+               $self->scan_core;
+            };
+        }
+        $self->try_bug_report();
+    }';
     die "failed: $@" if $@;
 
 }
@@ -463,6 +477,11 @@ sub pre_configure { }
 sub configure {
     my $self = shift;
 
+    if ($self->{opts}->{'no-httpd'}) {
+        warning "skipping httpd configuration";
+        return;
+    }
+
     # create the conf dir as early as possible
     $self->{test_config}->prepare_t_conf();
 
@@ -553,7 +572,8 @@ sub start {
     my $file = $server->debugger_file;
     if (-e $file and $opts->{'start-httpd'}) {
         if ($server->ping) {
-            warning "server is running under the debugger, defaulting to -run";
+            warning "server is running under the debugger, " .
+                "defaulting to -run";
             $opts->{'start-httpd'} = $opts->{'stop-httpd'} = 0;
         }
         else {
@@ -732,11 +752,11 @@ sub run {
 
     $self->die_on_invalid_args;
 
-    $self->start;
+    $self->start unless $self->{opts}->{'no-httpd'};
 
     $self->run_tests;
 
-    $self->stop;
+    $self->stop unless $self->{opts}->{'no-httpd'};
 }
 
 # make it easy to move the whole distro w/o running
@@ -822,7 +842,8 @@ sub scan_core_incremental {
             next unless -f;
             next unless /$core_pat/o;
             my $core = catfile $vars->{t_dir}, $_;
-            next if exists $core_files{$core} && $core_files{$core} == -M $core;
+            next if exists $core_files{$core} &&
+                $core_files{$core} == -M $core;
             $core_files{$core} = -M $core;
             push @cores, $core;
         }
@@ -975,7 +996,8 @@ sub run_root_fs_test {
     # setgroups() call as explained in perlvar.pod)
     my $groups = "$gid $gid";
     $( = $) = $groups;
-    die "failed to change gid to $gid" unless $( eq $groups && $) eq $groups;
+    die "failed to change gid to $gid"
+        unless $( eq $groups && $) eq $groups;
 
     # only now can change uid and euid
     $< = $> = $uid+0;
@@ -1110,7 +1132,8 @@ sub opt_ping {
     my $server = $test_config->server;
     my $pid = $server->ping;
     my $name = $server->{name};
-    my $exit = not $self->{opts}->{'run-tests'}; #support t/TEST -ping=block -run ...
+    # support t/TEST -ping=block -run ...
+    my $exit = not $self->{opts}->{'run-tests'};
 
     if ($pid) {
         if ($pid == -1) {
@@ -1409,8 +1432,8 @@ sub custom_config_save {
     return 0 unless $vars->{httpd} or $Apache::TestConfigData::vars->{httpd}
         or          $vars->{apxs}  or $Apache::TestConfigData::vars->{apxs};
 
-    # it doesn't matter how these vars were set (httpd may or may not get set
-    # using the path to apxs, w/o an explicit -httpd value)
+    # it doesn't matter how these vars were set (httpd may or may not
+    # get set using the path to apxs, w/o an explicit -httpd value)
     for (@data_vars_must) {
         next unless my $var = $vars->{$_} || $conf_opts->{$_};
         $config_dump .= qq{    '$_' => '$var',\n};
@@ -1418,7 +1441,7 @@ sub custom_config_save {
 
     # save these vars only if they were explicitly set via command line
     # options. For example if someone builds A-T as user 'foo', then
-    # installs it as root and we save it, all users will now try to 
+    # installs it as root and we save it, all users will now try to
     # configure under that user 'foo' which won't quite work.
     for (@data_vars_opt) {
         next unless my $var = $conf_opts->{$_};
