@@ -35,7 +35,7 @@ $VERSION = '0.01';
 @ISA     = qw(Exporter);
 
 @EXPORT = qw(t_cmp t_debug t_append_file t_write_file t_open_file
-    t_mkdir t_rmtree t_is_equal
+    t_mkdir t_rmtree t_is_equal t_filepath_cmp
     t_server_log_error_is_expected t_server_log_warn_is_expected
     t_client_log_error_is_expected t_client_log_warn_is_expected
 );
@@ -49,17 +49,97 @@ $VERSION = '0.01';
 use constant HAS_DUMPER => eval { $] >= 5.6 && require Data::Dumper; };
 use constant INDENT     => 4;
 
-sub t_cmp {
+# because of the prototype and recursive call to itself a forward
+# declaration is needed
+sub t_is_equal ($$);
+
+# compare any two datastructures (must pass references for non-scalars)
+# undef()'s are valid args
+sub t_is_equal ($$) {
+    my ($a, $b) = @_;
+    return 0 unless @_ == 2;
+
+    # this was added in Apache::Test::VERSION 1.12 - remove deprecated
+    # logic sometime around 1.15 or mid September, 2004.
+    if (UNIVERSAL::isa($a, 'Regexp')) {
+        my @warning = ("WARNING!!! t_is_equal() argument order has changed.",
+                       "use of a regular expression as the first argument",
+                       "is deprecated.  support will be removed soon.");
+        t_debug(@warning);
+        ($a, $b) = ($b, $a);
+    }
+
+    if (defined $a && defined $b) {
+        my $ref_a = ref $a;
+        my $ref_b = ref $b;
+        if (!$ref_a && !$ref_b) {
+            return $a eq $b;
+        }
+        elsif ($ref_a eq 'ARRAY' && $ref_b eq 'ARRAY') {
+            return 0 unless @$a == @$b;
+            for my $i (0..$#$a) {
+                t_is_equal($a->[$i], $b->[$i]) || return 0;
+            }
+        }
+        elsif ($ref_a eq 'HASH' && $ref_b eq 'HASH') {
+            return 0 unless (keys %$a) == (keys %$b);
+            for my $key (sort keys %$a) {
+                return 0 unless exists $b->{$key};
+                t_is_equal($a->{$key}, $b->{$key}) || return 0;
+            }
+        }
+        elsif ($ref_b eq 'Regexp') {
+            return $a =~ $b;
+        }
+        else {
+            # try to compare the references
+            return $a eq $b;
+        }
+    }
+    else {
+        # undef == undef! a valid test
+        return (defined $a || defined $b) ? 0 : 1;
+    }
+    return 1;
+}
+
+
+
+sub t_cmp ($$;$) {
     Carp::carp(join(":", (caller)[1..2]) . 
-        ' usage: $res = t_cmp($expected, $received, [$comment])')
+        ' usage: $res = t_cmp($received, $expected, [$comment])')
             if @_ < 2 || @_ > 3;
 
+    my ($received, $expected) = @_;
+
+    # this was added in Apache::Test::VERSION 1.12 - remove deprecated
+    # logic sometime around 1.15 or mid September, 2004.
+    if (UNIVERSAL::isa($_[0], 'Regexp')) {
+        my @warning = ("WARNING!!! t_cmp() argument order has changed.",
+                       "use of a regular expression as the first argument",
+                       "is deprecated.  support will be removed soon.");
+        t_debug(@warning);
+        ($received, $expected) = ($expected, $received);
+    }
+
     t_debug("testing : " . pop) if @_ == 3;
-    t_debug("expected: " . struct_as_string(0, $_[0]));
-    t_debug("received: " . struct_as_string(0, $_[1]));
-    return t_is_equal(@_);
+    t_debug("received: " . struct_as_string(0, $received));
+    t_debug("expected: " . struct_as_string(0, $expected));
+    return t_is_equal($received, $expected);
 }
 
+# Essentially t_cmp, but on Win32, first converts pathnames
+# to their DOS long name.
+sub t_filepath_cmp ($$;$) {
+    my @a = (shift, shift);
+    if (Apache::TestConfig::WIN32) {
+        $a[0] = Win32::GetLongPathName($a[0]) if defined $a[0];
+        $a[1] = Win32::GetLongPathName($a[1]) if defined $a[1];
+    }
+    return @_ == 1 ? t_cmp($a[0], $a[1], $_[0]) : t_cmp($a[0], $a[1]);
+}
+
+
 *expand = HAS_DUMPER ?
     sub { map { ref $_ ? Data::Dumper::Dumper($_) : $_ } @_ } :
     sub { @_ };
@@ -245,49 +325,20 @@ sub struct_as_string{
     }
 }
 
-# compare any two datastructures (must pass references for non-scalars)
-# undef()'s are valid args
-sub t_is_equal {
-    my ($a, $b) = @_;
-    return 0 unless @_ == 2;
+my $banner_format = 
+    "\n*** The following %s expected and harmless ***\n";
 
-    if (defined $a && defined $b) {
-        my $ref_a = ref $a;
-        my $ref_b = ref $b;
-        if (!$ref_a && !$ref_b) {
-            return $a eq $b;
-        }
-        elsif ($ref_a eq 'ARRAY' && $ref_b eq 'ARRAY') {
-            return 0 unless @$a == @$b;
-            for my $i (0..$#$a) {
-                t_is_equal($a->[$i], $b->[$i]) || return 0;
-            }
-        }
-        elsif ($ref_a eq 'HASH' && $ref_b eq 'HASH') {
-            return 0 unless (keys %$a) == (keys %$b);
-            for my $key (sort keys %$a) {
-                return 0 unless exists $b->{$key};
-                t_is_equal($a->{$key}, $b->{$key}) || return 0;
-            }
-        }
-        elsif ($ref_a eq 'Regexp') {
-            return $b =~ $a;
-        }
-        else {
-            # try to compare the references
-            return $a eq $b;
-        }
-    }
-    else {
-        # undef == undef! a valid test
-        return (defined $a || defined $b) ? 0 : 1;
-    }
-    return 1;
+sub is_expected_banner {
+    my $type  = shift;
+    my $count = @_ ? shift : 1;
+    sprintf $banner_format, $count == 1
+        ? "$type entry is"
+        : "$count $type entries are";
 }
 
-my $banner_format = 
-    "\n*** The following %s entry is expected and it is harmless ***\n";
-sub t_server_log_is_expected { printf STDERR $banner_format, $_[0]; }
+sub t_server_log_is_expected {
+    print STDERR is_expected_banner(@_);
+}
 
 sub t_client_log_is_expected {
     my $vars = Apache::Test::config()->{vars};
@@ -296,14 +347,14 @@ sub t_client_log_is_expected {
     my $fh = Symbol::gensym();
     open $fh, ">>$log_file" or die "Can't open $log_file: $!";
     my $oldfh = select($fh); $| = 1; select($oldfh);
-    printf $fh $banner_format, $_[0];
+    print $fh is_expected_banner(@_);
     close $fh;
 }
 
-sub t_server_log_error_is_expected { t_server_log_is_expected("error");}
-sub t_server_log_warn_is_expected  { t_server_log_is_expected("warn"); }
-sub t_client_log_error_is_expected { t_client_log_is_expected("error");}
-sub t_client_log_warn_is_expected  { t_client_log_is_expected("warn"); }
+sub t_server_log_error_is_expected { t_server_log_is_expected("error", @_);}
+sub t_server_log_warn_is_expected  { t_server_log_is_expected("warn", @_); }
+sub t_client_log_error_is_expected { t_client_log_is_expected("error", @_);}
+sub t_client_log_warn_is_expected  { t_client_log_is_expected("warn", @_); }
 
 END {
     # remove files that were created via this package
@@ -376,7 +427,7 @@ directories and files at the end of the test.
 
 =item t_cmp()
 
-  t_cmp($expected, $received, $comment);
+  t_cmp($received, $expected, $comment);
 
 t_cmp() prints the values of I<$comment>, I<$expected> and
 I<$received>. e.g.:
@@ -414,9 +465,9 @@ datastructures can be deeply nested. For example you can compare:
         "hash of array of hashes");
 
 You can also compare the second argument against the first as a
-regex. Use the C<qr//> function in the first argument. For example:
+regex. Use the C<qr//> function in the second argument. For example:
 
-  t_cmp(qr/^abc/, "abcd", "regex compare");
+  t_cmp("abcd", qr/^abc/, "regex compare");
 
 will do:
 
@@ -424,6 +475,17 @@ will do:
 
 This function is exported by default.
 
+=item t_filepath_cmp()
+
+This function is used to compare two filepaths via t_cmp().
+For non-Win32, it simply uses t_cmp() for the comparison,
+but for Win32, Win32::GetLongPathName() is invoked to convert
+the first two arguments to their DOS long pathname. This is useful
+when there is a possibility the two paths being compared
+are not both represented by their long or short pathname.
+
+This function is exported by default.
+
 =item t_debug()
 
   t_debug("testing feature foo");
@@ -544,10 +606,10 @@ exactly the same, otherwise 0. The datastructures can be nested
 hashes, arrays, scalars, undefs or a combination of any of these.  See
 t_cmp() for an example.
 
-If C<$a> is a regex reference, the regex comparison C<$b =~ $a> is
+If C<$b> is a regex reference, the regex comparison C<$a =~ $b> is
 performed. For example:
 
-  t_is_equal(qr{^Apache}, $server_version);
+  t_is_equal($server_version, qr{^Apache});
 
 If comparing non-scalars make sure to pass the references to the
 datastructures.
@@ -577,9 +639,18 @@ function can be used as:
 
 After running this handler the I<error_log> file will include:
 
-  *** The following error entry is expected and it is harmless ***
+  *** The following error entry is expected and harmless ***
   [Tue Apr 01 14:00:21 2003] [error] failed because ...
 
+When more than one entry is expected, an optional numerical argument,
+indicating how many entries to expect, can be passed. For example:
+
+  t_server_log_error_is_expected(2);
+
+will generate:
+
+  *** The following 2 error entries are expected and harmless ***
+
 If the error is generated at compile time, the logging must be done in
 the BEGIN block at the very beginning of the file:
 
@@ -592,7 +663,7 @@ the BEGIN block at the very beginning of the file:
 After attempting to run this handler the I<error_log> file will
 include:
 
-  *** The following error entry is expected and it is harmless ***
+  *** The following error entry is expected and harmless ***
   [Tue Apr 01 14:04:49 2003] [error] Can't locate "DOES_NOT_exist.pm"
   in @INC (@INC contains: ...
 
@@ -637,10 +708,19 @@ For example the following client script fails to find the handler:
 After running this test the I<error_log> file will include an entry
 similar to the following snippet:
 
-  *** The following error entry is expected and it is harmless ***
+  *** The following error entry is expected and harmless ***
   [Tue Apr 01 14:02:55 2003] [error] [client 127.0.0.1] 
   File does not exist: /tmp/test/t/htdocs/error
 
+When more than one entry is expected, an optional numerical argument,
+indicating how many entries to expect, can be passed. For example:
+
+  t_client_log_error_is_expected(2);
+
+will generate:
+
+  *** The following 2 error entries are expected and harmless ***
+
 This function is exported by default.
 
 =item t_client_log_warn_is_expected()
